/* Copyright 2015, Michele Santullo
 * This file is part of DoorKeeper.
 *
 * DoorKeeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * DoorKeeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with DoorKeeper.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef id0ADBCC15BA574485BF3267254090D99B
#define id0ADBCC15BA574485BF3267254090D99B

#include "doorkeeper/primitivetypes.hpp"
#include "doorkeeper/components/tileiterator.hpp"
#include "doorkeeper/components/layer.hpp"
#include <cstdint>

namespace dk {
	template <typename PixConv>
	class Tyler;

	template <typename T, typename PixConv>
	class Layer;

	template <typename PixConv>
	class Viewport {
	public:
		enum {
			dimensions = PixConv::dimensions
		};

		typedef Vector<dimensions> coords;

		Viewport ( const Viewport& parOther ) = default;
		explicit Viewport ( Tyler<PixConv>& parTyler );
		Viewport ( Tyler<PixConv>& parTyler, const coords& parSize, const coords& parPos );
		~Viewport ( void ) noexcept = default;

		Viewport& operator= ( const Viewport& ) = default;

		void setSize ( const coords& parSize );
		void setFrom ( const coords& parFrom );
		void setFromSize ( const coords& parFrom, const coords& parSize );
		const coords& count ( void ) const { return m_size; }

		template <typename T>
		typename Layer<T, PixConv>::iterator begin ( Layer<T, PixConv>& parLayer ) const;
		template <typename T>
		typename Layer<T, PixConv>::iterator end ( Layer<T, PixConv>& parLayer ) const;
		template <typename T> inline
		typename Layer<T, PixConv>::const_iterator begin ( const Layer<T, PixConv>& parLayer ) const a_always_inline;
		template <typename T> inline
		typename Layer<T, PixConv>::const_iterator end ( const Layer<T, PixConv>& parLayer ) const a_always_inline;
		template <typename T>
		typename Layer<T, PixConv>::const_iterator cbegin ( const Layer<T, PixConv>& parLayer ) const;
		template <typename T>
		typename Layer<T, PixConv>::const_iterator cend ( const Layer<T, PixConv>& parLayer ) const;

	private:
		coords m_size;
		coords m_position;
		Tyler<PixConv>& m_tyler;
	};
} //namespace dk

#include "doorkeeper/implem/viewport.inl"

#endif
